<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

## Appium

**Extends Webdriver**

Appium helper extends [Webriver][1] helper.
 It supports all browser methods and also includes special methods for mobile apps testing.
 You can use this helper to test Web on desktop and mobile devices and mobile apps.

## Appium Installation

Appium is an open source test automation framework for use with native, hybrid and mobile web apps that implements the WebDriver protocol.
It allows you to run Selenium tests on mobile devices and also test native, hybrid and mobile web apps.

Download and install [Appium][2]

```sh
npm install -g appium
```

Launch the daemon: `appium`

## Helper configuration

This helper should be configured in codecept.json or codecept.conf.js

-   `app`: Application path. Local path or remote URL to an .ipa or .apk file, or a .zip containing one of these. Alias to desiredCapabilities.appPackage
-   `host`: (default: 'localhost') Appium host
-   `port`: (default: '4723') Appium port
-   `platform`: (Android or IOS), which mobile OS to use; alias to desiredCapabilities.platformName
-   `restart`: restart browser or app between tests (default: true), if set to false cookies will be cleaned but browser window will be kept and for apps nothing will be changed.
-   `desiredCapabilities`: \[], Appium capabilities, see below
    -   `platformName` - Which mobile OS platform to use
    -   `appPackage` - Java package of the Android app you want to run
    -   `appActivity` - Activity name for the Android activity you want to launch from your package.
    -   `deviceName`: The kind of mobile device or emulator to use
    -   `platformVersion`: Mobile OS version
    -   `app` - The absolute local path or remote http URL to an .ipa or .apk file, or a .zip containing one of these. Appium will attempt to install this app binary on the appropriate device first.
    -   `browserName`: Name of mobile web browser to automate. Should be an empty string if automating an app instead.

Example Android App:

```js
{
  helpers: {
      Appium: {
          platform: "Android",
          desiredCapabilities: {
              appPackage: "com.example.android.myApp",
              appActivity: "MainActivity",
              deviceName: "OnePlus3",
              platformVersion: "6.0.1"
          }
      }
    }
}
```

Example iOS Mobile Web with local Appium:

```js
{
helpers: {
  Appium: {
    platform: "iOS",
    url: "https://the-internet.herokuapp.com/",
    desiredCapabilities: {
      deviceName: "iPhone X",
      platformVersion: "12.0",
      browserName: "safari"
    }
  }
}
}
```

Example iOS Mobile Web on BrowserStack:

```js
{
helpers: {
  Appium: {
    host: "hub-cloud.browserstack.com",
    port: 4444,
    user: process.env.BROWSERSTACK_USER,
    key: process.env.BROWSERSTACK_KEY,
    platform: "iOS",
    url: "https://the-internet.herokuapp.com/",
    desiredCapabilities: {
      realMobile: "true",
      device: "iPhone 8",
      os_version: "12",
      browserName: "safari"
    }
  }
}
}
```

Additional configuration params can be used from [https://github.com/appium/appium/blob/master/docs/en/writing-running-appium/caps.md][3]

## Access From Helpers

Receive a Appium client from a custom helper by accessing `browser` property:

```js
let browser = this.helpers['Appium'].browser
```

## Methods

### Parameters

-   `config`  

### runOnIOS

Execute code only on iOS

```js
I.runOnIOS(() => {
   I.click('//UIAApplication[1]/UIAWindow[1]/UIAButton[1]');
   I.see('Hi, IOS', '~welcome');
});
```

Additional filter can be applied by checking for capabilities.
For instance, this code will be executed only on iPhone 5s:

```js
I.runOnIOS({deviceName: 'iPhone 5s'},() => {
   // ...
});
```

Also capabilities can be checked by a function.

```js
I.runOnAndroid((caps) => {
   // caps is current config of desiredCapabiliites
   return caps.platformVersion >= 6
},() => {
   // ...
});
```

#### Parameters

-   `caps` **any** 
-   `fn` **any** 

### runOnAndroid

Execute code only on Android

```js
I.runOnAndroid(() => {
   I.click('io.selendroid.testapp:id/buttonTest');
});
```

Additional filter can be applied by checking for capabilities.
For instance, this code will be executed only on Android 6.0:

```js
I.runOnAndroid({platformVersion: '6.0'},() => {
   // ...
});
```

Also capabilities can be checked by a function.
In this case, code will be executed only on Android >= 6.

```js
I.runOnAndroid((caps) => {
   // caps is current config of desiredCapabiliites
   return caps.platformVersion >= 6
},() => {
   // ...
});
```

#### Parameters

-   `caps` **any** 
-   `fn` **any** 

### runInWeb

Execute code only in Web mode.

```js
I.runInWeb(() => {
   I.waitForElement('#data');
   I.seeInCurrentUrl('/data');
});
```

#### Parameters

-   `fn` **any** 

### seeAppIsInstalled

Check if an app is installed.

```js
I.seeAppIsInstalled("com.example.android.apis");
```

#### Parameters

-   `bundleId` **[string][4]** String  ID of bundled appAppium: support only Android

### seeAppIsNotInstalled

Check if an app is not installed.

```js
I.seeAppIsNotInstalled("com.example.android.apis");
```

#### Parameters

-   `bundleId` **[string][4]** String  ID of bundled appAppium: support only Android

### installApp

Install an app on device.

```js
I.installApp('/path/to/file.apk');
```

#### Parameters

-   `path` **[string][4]** path to apk fileAppium: support only Android

### removeApp

Remove an app from the device.

```js
I.removeApp('appName', 'com.example.android.apis');
```

#### Parameters

-   `appId` **[string][4]** 
-   `bundleId` **[string][4]** String  ID of bundleAppium: support only Android

### seeCurrentActivityIs

Check current activity on an Android device.

```js
I.seeCurrentActivityIs(".HomeScreenActivity")
```

#### Parameters

-   `currentActivity` **[string][4]** Appium: support only Android

### seeDeviceIsLocked

Check whether the device is locked.

```js
I.seeDeviceIsLocked();
```

Appium: support only Android

### seeDeviceIsUnlocked

Check whether the device is not locked.

```js
I.seeDeviceIsUnlocked();
```

Appium: support only Android

### seeOrientationIs

Check the device orientation

```js
I.seeOrientationIs('PORTRAIT');
I.seeOrientationIs('LANDSCAPE')
```

#### Parameters

-   `orientation` **(`"LANDSCAPE"` \| `"PORTRAIT"`)** LANDSCAPE or PORTRAITAppium: support Android and iOS

### setOrientation

Set a device orientation. Will fail, if app will not set orientation

```js
I.setOrientation('PORTRAIT');
I.setOrientation('LANDSCAPE')
```

#### Parameters

-   `orientation` **(`"LANDSCAPE"` \| `"PORTRAIT"`)** LANDSCAPE or PORTRAITAppium: support Android and iOS

### grabAllContexts

Get list of all available contexts

    let contexts = await I.grabAllContexts();

Appium: support Android and iOS

### grabContext

Retrieve current context

```js
let context = await I.grabContext();
```

Appium: support Android and iOS

### grabCurrentActivity

Get current device activity.

```js
let activity = await I.grabCurrentActivity();
```

Appium: support only Android

### grabNetworkConnection

Get information about the current network connection (Data/WIFI/Airplane).
The actual server value will be a number. However WebdriverIO additional
properties to the response object to allow easier assertions.

```js
let con = await I.grabNetworkConnection();
```

Appium: support only Android

### grabOrientation

Get current orientation.

```js
let orientation = await I.grabOrientation();
```

Appium: support Android and iOS

### grabSettings

Get all the currently specified settings.

```js
let settings = await I.grabSettings();
```

Appium: support Android and iOS

### \_switchToContext

Switch to the specified context.

#### Parameters

-   `context` **any** the context to switch to

### switchToWeb

Switches to web context.
If no context is provided switches to the first detected web context

```js
// switch to first web context
I.switchToWeb();

// or set the context explicitly
I.switchToWeb('WEBVIEW_io.selendroid.testapp');
```

#### Parameters

-   `context` **[string][4]?** 

### switchToNative

Switches to native context.
By default switches to NATIVE_APP context unless other specified.

```js
I.switchToNative();

// or set context explicitly
I.switchToNative('SOME_OTHER_CONTEXT');
```

#### Parameters

-   `context` **any**  (optional, default `null`)

### startActivity

Start an arbitrary Android activity during a session.

```js
I.startActivity('io.selendroid.testapp', '.RegisterUserActivity');
```

Appium: support only Android

#### Parameters

-   `appPackage`  
-   `appActivity`  

### setNetworkConnection

Set network connection mode.

-   airplane mode
-   wifi mode
-   data data

```js
I.setNetworkConnection(0) // airplane mode off, wifi off, data off
I.setNetworkConnection(1) // airplane mode on, wifi off, data off
I.setNetworkConnection(2) // airplane mode off, wifi on, data off
I.setNetworkConnection(4) // airplane mode off, wifi off, data on
I.setNetworkConnection(6) // airplane mode off, wifi on, data on
```

See corresponding [webdriverio reference][5].

Appium: support only Android

#### Parameters

-   `value`  

### setSettings

Update the current setting on the device

```js
I.setSettings({cyberdelia: 'open'});
```

#### Parameters

-   `settings` **[object][6]** objectAppium: support Android and iOS

### hideDeviceKeyboard

Hide the keyboard.

```js
// taps outside to hide keyboard per default
I.hideDeviceKeyboard();
I.hideDeviceKeyboard('tapOutside');

// or by pressing key
I.hideDeviceKeyboard('pressKey', 'Done');
```

#### Parameters

-   `strategy` **(`"tapOutside"` \| `"pressKey"`)** desired strategy to close keyboard (‘tapOutside’ or ‘pressKey’)Appium: support Android and iOS
-   `key`  

### sendDeviceKeyEvent

Send a key event to the device.
List of keys: [https://developer.android.com/reference/android/view/KeyEvent.html][7]

```js
I.sendDeviceKeyEvent(3);
```

#### Parameters

-   `keyValue` **[number][8]** Device specific key valueAppium: support only Android

### openNotifications

Open the notifications panel on the device.

```js
I.openNotifications();
```

Appium: support only Android

### makeTouchAction

The Touch Action API provides the basis of all gestures that can be
automated in Appium. At its core is the ability to chain together ad hoc
individual actions, which will then be applied to an element in the
application on the device.
[See complete documentation][9]

```js
I.makeTouchAction("~buttonStartWebviewCD", 'tap');
```

Appium: support Android and iOS

#### Parameters

-   `locator`  
-   `action`  

### tap

Taps on element.

```js
I.tap("~buttonStartWebviewCD");
```

Shortcut for `makeTouchAction`

#### Parameters

-   `locator` **any** 

### swipe

Perform a swipe on the screen or an element.

```js
let locator = "#io.selendroid.testapp:id/LinearLayout1";
I.swipe(locator, 800, 1200, 1000);
```

[See complete reference][10]

#### Parameters

-   `locator` **([string][4] \| [object][6])** 
-   `xoffset` **[number][8]** 
-   `yoffset` **[number][8]** 
-   `speed` **[number][8]** (optional), 1000 by defaultAppium: support Android and iOS (optional, default `1000`)

### performSwipe

Perform a swipe on the screen.

```js
I.performswipe(100,200);
```

#### Parameters

-   `from` **[number][8]** 
-   `to` **[number][8]** Appium: support Android and iOS

### swipeDown

Perform a swipe down on an element.

```js
let locator = "#io.selendroid.testapp:id/LinearLayout1";
I.swipeDown(locator); // simple swipe
I.swipeDown(locator, 500); // set speed
I.swipeDown(locator, 1200, 1000); // set offset and speed
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** 
-   `yoffset` **[number][8]?** (optional) (optional, default `1000`)
-   `speed` **[number][8]** (optional), 1000 by defaultAppium: support Android and iOS (optional, default `1000`)

### swipeLeft

Perform a swipe left on an element.

```js
let locator = "#io.selendroid.testapp:id/LinearLayout1";
I.swipeLeft(locator); // simple swipe
I.swipeLeft(locator, 500); // set speed
I.swipeLeft(locator, 1200, 1000); // set offset and speed
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** 
-   `xoffset` **[number][8]?** (optional) (optional, default `1000`)
-   `speed` **[number][8]** (optional), 1000 by defaultAppium: support Android and iOS (optional, default `1000`)

### swipeRight

Perform a swipe right on an element.

```js
let locator = "#io.selendroid.testapp:id/LinearLayout1";
I.swipeRight(locator); // simple swipe
I.swipeRight(locator, 500); // set speed
I.swipeRight(locator, 1200, 1000); // set offset and speed
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** 
-   `xoffset` **[number][8]?** (optional) (optional, default `1000`)
-   `speed` **[number][8]** (optional), 1000 by defaultAppium: support Android and iOS (optional, default `1000`)

### swipeUp

Perform a swipe up on an element.

```js
let locator = "#io.selendroid.testapp:id/LinearLayout1";
I.swipeUp(locator); // simple swipe
I.swipeUp(locator, 500); // set speed
I.swipeUp(locator, 1200, 1000); // set offset and speed
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** 
-   `yoffset` **[number][8]?** (optional) (optional, default `1000`)
-   `speed` **[number][8]** (optional), 1000 by defaultAppium: support Android and iOS (optional, default `1000`)

### swipeTo

Perform a swipe in selected direction on an element to searchable element.

```js
I.swipeTo(
 "android.widget.CheckBox", // searchable element
 "//android.widget.ScrollView/android.widget.LinearLayout", // scroll element
  "up", // direction
   30,
   100,
   500);
```

#### Parameters

-   `searchableLocator` **[string][4]** 
-   `scrollLocator` **[string][4]** 
-   `direction` **[string][4]** 
-   `timeout` **[number][8]** 
-   `offset` **[number][8]** 
-   `speed` **[number][8]** Appium: support Android and iOS

### touchPerform

Performs a specific touch action.
The action object need to contain the action name, x/y coordinates

```js
I.touchPerform([{
    action: 'press',
    options: {
      x: 100,
      y: 200
    }
}, {action: 'release'}])

I.touchPerform([{
   action: 'tap',
   options: {
       element: '1', // json web element was queried before
       x: 10,   // x offset
       y: 20,   // y offset
       count: 1 // number of touches
   }
}]);
```

Appium: support Android and iOS

#### Parameters

-   `actions`  

### pullFile

Pulls a file from the device.

```js
I.pullFile('/storage/emulated/0/DCIM/logo.png', 'my/path');
// save file to output dir
I.pullFile('/storage/emulated/0/DCIM/logo.png', output_dir);
```

Appium: support Android and iOS

#### Parameters

-   `path`  
-   `dest`  

### shakeDevice

Perform a shake action on the device.

```js
I.shakeDevice();
```

Appium: support only iOS

### rotate

Perform a rotation gesture centered on the specified element.

```js
I.rotate(120, 120)
```

See corresponding [webdriverio reference][11].

Appium: support only iOS

#### Parameters

-   `x`  
-   `y`  
-   `duration`  
-   `radius`  
-   `rotation`  
-   `touchCount`  

### setImmediateValue

Set immediate value in app.

See corresponding [webdriverio reference][12].

Appium: support only iOS

#### Parameters

-   `id`  
-   `value`  

### simulateTouchId

Simulate Touch ID with either valid (match == true) or invalid (match == false) fingerprint.

```js
I.touchId(); // simulates valid fingerprint
I.touchId(true); // simulates valid fingerprint
I.touchId(false); // simulates invalid fingerprint
```

Appium: support only iOS
TODO: not tested

#### Parameters

-   `match`  

### closeApp

Close the given application.

```js
I.closeApp();
```

Appium: support only iOS

### appendField

Appends text to a input field or textarea.
Field is located by name, label, CSS or XPath

```js
I.appendField('#myTextField', 'appended');
```

#### Parameters

-   `field` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator
-   `value` **[string][4]** text value to append.

### checkOption

Selects a checkbox or radio button.
Element is located by label or name or CSS or XPath.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
I.checkOption('#agree');
I.checkOption('I Agree to Terms and Conditions');
I.checkOption('agree', '//form');
```

#### Parameters

-   `field` **([string][4] \| [object][6])** checkbox located by label | name | CSS | XPath | strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element located by CSS | XPath | strict locator. (optional, default `null`)

### click

Perform a click on a link or a button, given by a locator.
If a fuzzy locator is given, the page will be searched for a button, link, or image matching the locator string.
For buttons, the "value" attribute, "name" attribute, and inner text are searched. For links, the link text is searched.
For images, the "alt" attribute and inner text of any parent links are searched.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
// simple link
I.click('Logout');
// button of form
I.click('Submit');
// CSS button
I.click('#form input[type=submit]');
// XPath
I.click('//form/*[@type=submit]');
// link in context
I.click('Logout', '#nav');
// using strict locator
I.click({css: 'nav a.login'});
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element to search in CSS|XPath|Strict locator. (optional, default `null`)

### dontSeeCheckboxIsChecked

Verifies that the specified checkbox is not checked.

```js
I.dontSeeCheckboxIsChecked('#agree'); // located by ID
I.dontSeeCheckboxIsChecked('I agree to terms'); // located by label
I.dontSeeCheckboxIsChecked('agree'); // located by name
```

#### Parameters

-   `field` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator.

### dontSeeElement

Opposite to `seeElement`. Checks that element is not visible (or in DOM)

```js
I.dontSeeElement('.modal'); // modal is not shown
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|Strict locator.

### dontSeeInField

Checks that value of input field or textarea doesn't equal to given value
Opposite to `seeInField`.

```js
I.dontSeeInField('email', 'user@user.com'); // field by name
I.dontSeeInField({ css: 'form input.email' }, 'user@user.com'); // field by CSS
```

#### Parameters

-   `field` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator.
-   `value` **[string][4]** value to check.

### dontSee

Opposite to `see`. Checks that a text is not present on a page.
Use context parameter to narrow down the search.

```js
I.dontSee('Login'); // assume we are already logged in.
I.dontSee('Login', '.nav'); // no login inside .nav element
```

#### Parameters

-   `text` **[string][4]** which is not present.
-   `context` **([string][4] \| [object][6])?** (optional) element located by CSS|XPath|strict locator in which to perfrom search. (optional, default `null`)

### fillField

Fills a text field or textarea, after clearing its value, with the given string.
Field is located by name, label, CSS, or XPath.

```js
// by label
I.fillField('Email', 'hello@world.com');
// by name
I.fillField('password', secret('123456'));
// by CSS
I.fillField('form#login input[name=username]', 'John');
// or by strict locator
I.fillField({css: 'form#login input[name=username]'}, 'John');
```

#### Parameters

-   `field` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator.
-   `value` **[string][4]** text value to fill.

### grabTextFrom

Retrieves a text from an element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let pin = await I.grabTextFrom('#pin');
```

If multiple elements found returns an array of texts.

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.

Returns **[Promise][13]&lt;([string][4] \| [Array][14]&lt;[string][4]>)>** attribute value

### grabValueFrom

Retrieves a value from a form element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let email = await I.grabValueFrom('input[name=email]');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** field located by label|name|CSS|XPath|strict locator.

Returns **[Promise][13]&lt;[string][4]>** attribute value

### scrollIntoView

Scroll element into viewport.

```js
I.scrollIntoView('#submit');
I.scrollIntoView('#submit', true);
I.scrollIntoView('#submit', { behavior: "smooth", block: "center", inline: "center" });
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.
-   `scrollIntoViewOptions`  
-   `alignToTop` **([boolean][15] \| [object][6])** (optional) or scrollIntoViewOptions (optional), see [https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView][16].Supported only for web testing

### seeCheckboxIsChecked

Verifies that the specified checkbox is checked.

```js
I.seeCheckboxIsChecked('Agree');
I.seeCheckboxIsChecked('#agree'); // I suppose user agreed to terms
I.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
```

#### Parameters

-   `field` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator.

### seeElement

Checks that a given Element is visible
Element is located by CSS or XPath.

```js
I.seeElement('#modal');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.

### seeInField

Checks that the given input field or textarea equals to given value.
For fuzzy locators, fields are matched by label text, the "name" attribute, CSS, and XPath.

```js
I.seeInField('Username', 'davert');
I.seeInField({css: 'form textarea'},'Type your comment here');
I.seeInField('form input[type=hidden]','hidden_value');
I.seeInField('#searchform input','Search');
```

#### Parameters

-   `field` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator.
-   `value` **[string][4]** value to check.

### see

Checks that a page contains a visible text.
Use context parameter to narrow down the search.

```js
I.see('Welcome'); // text welcome on a page
I.see('Welcome', '.content'); // text inside .content div
I.see('Register', {css: 'form.register'}); // use strict locator
```

#### Parameters

-   `text` **[string][4]** expected on page.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element located by CSS|Xpath|strict locator in which to search for text. (optional, default `null`)

### selectOption

Selects an option in a drop-down select.
Field is searched by label | name | CSS | XPath.
Option is selected by visible text or by value.

```js
I.selectOption('Choose Plan', 'Monthly'); // select by label
I.selectOption('subscription', 'Monthly'); // match option by text
I.selectOption('subscription', '0'); // or by value
I.selectOption('//form/select[@name=account]','Premium');
I.selectOption('form select[name=account]', 'Premium');
I.selectOption({css: 'form select[name=account]'}, 'Premium');
```

Provide an array for the second argument to select multiple options.

```js
I.selectOption('Which OS do you use?', ['Android', 'iOS']);
```

#### Parameters

-   `select` **([string][4] \| [object][6])** field located by label|name|CSS|XPath|strict locator.
-   `option` **([string][4] \| [Array][14]&lt;any>)** visible text or value of option.Supported only for web testing

### waitForElement

Waits for element to be present on page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForElement('.btn.continue');
I.waitForElement('.btn.continue', 5); // wait for 5 secs
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `sec` **[number][8]?** (optional, `1` by default) time in seconds to wait (optional, default `null`)

### waitForVisible

Waits for an element to become visible on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForVisible('#popup');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `sec` **[number][8]** (optional, `1` by default) time in seconds to wait (optional, default `1`)

### waitForInvisible

Waits for an element to be removed or become invisible on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForInvisible('#popup');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `sec` **[number][8]** (optional, `1` by default) time in seconds to wait (optional, default `1`)

### waitForText

Waits for a text to appear (by default waits for 1sec).
Element can be located by CSS or XPath.
Narrow down search results by providing context.

```js
I.waitForText('Thank you, form has been submitted');
I.waitForText('Thank you, form has been submitted', 5, '#modal');
```

#### Parameters

-   `text` **[string][4]** to wait for.
-   `sec` **[number][8]** (optional, `1` by default) time in seconds to wait (optional, default `1`)
-   `context` **([string][4] \| [object][6])?** (optional) element located by CSS|XPath|strict locator. (optional, default `null`)

### \_isShadowLocator

Check if locator is type of "Shadow"

#### Parameters

-   `locator` **[object][6]** 

### \_locateShadow

Locate Element within the Shadow Dom

#### Parameters

-   `locator` **[object][6]** 

### \_smartWait

Smart Wait to locate an element

#### Parameters

-   `locator` **[object][6]** 

### \_locate

Get elements by different locator types, including strict locator.
Should be used in custom helpers:

```js
this.helpers['WebDriver']._locate({name: 'password'}).then //...
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `smartWait`   (optional, default `false`)

### \_locateCheckable

Find a checkbox by providing human readable text:

```js
this.helpers['WebDriver']._locateCheckable('I agree with terms and conditions').then // ...
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.

### \_locateClickable

Find a clickable element by providing human readable text:

```js
const els = await this.helpers.WebDriver._locateClickable('Next page');
const els = await this.helpers.WebDriver._locateClickable('Next page', '.pages');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `context`  

### \_locateFields

Find field elements by providing human readable text:

```js
this.helpers['WebDriver']._locateFields('Your email').then // ...
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.

### defineTimeout

Set [WebDriver timeouts][17] in realtime.

Timeouts are expected to be passed as object:

```js
I.defineTimeout({ script: 5000 });
I.defineTimeout({ implicit: 10000, pageLoad: 10000, script: 5000 });
```

#### Parameters

-   `timeouts` **WebdriverIO.Timeouts** WebDriver timeouts object.

### amOnPage

Opens a web page in a browser. Requires relative or absolute url.
If url starts with `/`, opens a web page of a site defined in `url` config parameter.

```js
I.amOnPage('/'); // opens main page of website
I.amOnPage('https://github.com'); // opens github
I.amOnPage('/login'); // opens a login page
```

#### Parameters

-   `url` **[string][4]** url path or global url.

### forceClick

Perform an emulated click on a link or a button, given by a locator.
Unlike normal click instead of sending native event, emulates a click with JavaScript.
This works on hidden, animated or inactive elements as well.

If a fuzzy locator is given, the page will be searched for a button, link, or image matching the locator string.
For buttons, the "value" attribute, "name" attribute, and inner text are searched. For links, the link text is searched.
For images, the "alt" attribute and inner text of any parent links are searched.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
// simple link
I.forceClick('Logout');
// button of form
I.forceClick('Submit');
// CSS button
I.forceClick('#form input[type=submit]');
// XPath
I.forceClick('//form/*[@type=submit]');
// link in context
I.forceClick('Logout', '#nav');
// using strict locator
I.forceClick({css: 'nav a.login'});
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element to search in CSS|XPath|Strict locator.{{ react }} (optional, default `null`)

### doubleClick

Performs a double-click on an element matched by link|button|label|CSS or XPath.
Context can be specified as second parameter to narrow search.

```js
I.doubleClick('Edit');
I.doubleClick('Edit', '.actions');
I.doubleClick({css: 'button.accept'});
I.doubleClick('.btn.edit');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element to search in CSS|XPath|Strict locator.{{ react }} (optional, default `null`)

### rightClick

Performs right click on a clickable element matched by semantic locator, CSS or XPath.

```js
// right click element with id el
I.rightClick('#el');
// right click link or button with text "Click me"
I.rightClick('Click me');
// right click button with text "Click me" inside .context
I.rightClick('Click me', '.context');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** clickable element located by CSS|XPath|strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element located by CSS|XPath|strict locator.{{ react }} (optional, default `null`)

### forceRightClick

Emulates right click on an element.
Unlike normal click instead of sending native event, emulates a click with JavaScript.
This works on hidden, animated or inactive elements as well.

If a fuzzy locator is given, the page will be searched for a button, link, or image matching the locator string.
For buttons, the "value" attribute, "name" attribute, and inner text are searched. For links, the link text is searched.
For images, the "alt" attribute and inner text of any parent links are searched.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
// simple link
I.forceRightClick('Menu');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element to search in CSS|XPath|Strict locator.{{ react }} (optional, default `null`)

### clearField

Clears a `<textarea>` or text `<input>` element's value.

```js
I.clearField('Email');
I.clearField('user[email]');
I.clearField('#email');
```

#### Parameters

-   `field`  
-   `editable` **([string][4] \| [object][6])** field located by label|name|CSS|XPath|strict locator.

### attachFile

Attaches a file to element located by label, name, CSS or XPath
Path to file is relative current codecept directory (where codecept.json or codecept.conf.js is located).
File will be uploaded to remote system (if tests are running remotely).

```js
I.attachFile('Avatar', 'data/avatar.jpg');
I.attachFile('form input[name=avatar]', 'data/avatar.jpg');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** field located by label|name|CSS|XPath|strict locator.
-   `pathToFile` **[string][4]** local file path relative to codecept.json config file.
    Appium: not tested

### uncheckOption

Unselects a checkbox or radio button.
Element is located by label or name or CSS or XPath.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
I.uncheckOption('#agree');
I.uncheckOption('I Agree to Terms and Conditions');
I.uncheckOption('agree', '//form');
```

#### Parameters

-   `field` **([string][4] \| [object][6])** checkbox located by label | name | CSS | XPath | strict locator.
-   `context` **([string][4]? | [object][6])** (optional, `null` by default) element located by CSS | XPath | strict locator.
    Appium: not tested (optional, default `null`)

### grabHTMLFrom

Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.
If more than one element is found - an array of HTMLs returned.

```js
let postHTML = await I.grabHTMLFrom('#post');
```

#### Parameters

-   `locator`  
-   `element` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.

Returns **[Promise][13]&lt;[string][4]>** HTML code for an element

### grabAttributeFrom

Retrieves an attribute from an element located by CSS or XPath and returns it to test.
An array as a result will be returned if there are more than one matched element.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let hint = await I.grabAttributeFrom('#tooltip', 'title');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `attr` **[string][4]** attribute name.

Returns **[Promise][13]&lt;[string][4]>** attribute value
Appium: can be used for apps only with several values ("contentDescription", "text", "className", "resourceId")

### seeTextEquals

Checks that text is equal to provided one.

```js
I.seeTextEquals('text', 'h1');
```

#### Parameters

-   `text` **[string][4]** element value to check.
-   `context` **([string][4] \| [object][6]?)** element located by CSS|XPath|strict locator. (optional, default `null`)

### seeElementInDOM

Checks that a given Element is present in the DOM
Element is located by CSS or XPath.

```js
I.seeElementInDOM('#modal');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.

### dontSeeElementInDOM

Opposite to `seeElementInDOM`. Checks that element is not on page.

```js
I.dontSeeElementInDOM('.nav'); // checks that element is not on page visible or not
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|Strict locator.

### seeInSource

Checks that the current page contains the given string in its raw source code.

```js
I.seeInSource('<h1>Green eggs &amp; ham</h1>');
```

#### Parameters

-   `text` **[string][4]** value to check.

### grabSource

Retrieves page source and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let pageSource = await I.grabSource();
```

Returns **[Promise][13]&lt;[string][4]>** source code

### grabBrowserLogs

Get JS log from browser. Log buffer is reset after each request.

```js
let logs = await I.grabBrowserLogs();
console.log(JSON.stringify(logs))
```

Returns **[Promise][13]&lt;([string][4] \| [undefined][18])>** 

### dontSeeInSource

Checks that the current page does not contains the given string in its raw source code.

```js
I.dontSeeInSource('<!--'); // no comments in source
```

#### Parameters

-   `text`  
-   `value` **[string][4]** to check.

### seeNumberOfElements

Asserts that an element appears a given number of times in the DOM.
Element is located by label or name or CSS or XPath.

```js
I.seeNumberOfElements('#submitBtn', 1);
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `num` **[number][8]** number of elements.{{ react }}

### seeNumberOfVisibleElements

Asserts that an element is visible a given number of times.
Element is located by CSS or XPath.

```js
I.seeNumberOfVisibleElements('.buttons', 3);
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `num` **[number][8]** number of elements.{{ react }}

### seeAttributesOnElements

Checks that all elements with given locator have given attributes.

```js
I.seeAttributesOnElements('//form', { method: "post"});
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.
-   `attributes` **[object][6]** attributes and their values to check.

### grabNumberOfVisibleElements

Grab number of visible elements by locator.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let numOfElements = await I.grabNumberOfVisibleElements('p');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.

Returns **[Promise][13]&lt;[number][8]>** number of visible elements

### scrollTo

Scrolls to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.scrollTo('footer');
I.scrollTo('#submit', 5, 5);
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.
-   `offsetX` **[number][8]** (optional, `0` by default) X-axis offset. (optional, default `0`)
-   `offsetY` **[number][8]** (optional, `0` by default) Y-axis offset. (optional, default `0`)

### moveCursorTo

Moves cursor to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.moveCursorTo('.tooltip');
I.moveCursorTo('#submit', 5,5);
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.
-   `xOffset`  
-   `yOffset`  
-   `offsetX` **[number][8]** (optional, `0` by default) X-axis offset. (optional, default `0`)
-   `offsetY` **[number][8]** (optional, `0` by default) Y-axis offset. (optional, default `0`)

### saveElementScreenshot

Saves screenshot of the specified locator to ouput folder (set in codecept.json or codecept.conf.js).
Filename is relative to output folder.

```js
I.saveElementScreenshot(`#submit`,'debug.png');
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `fileName` **[string][4]** file name to save.

### saveScreenshot

Saves a screenshot to ouput folder (set in codecept.json or codecept.conf.js).
Filename is relative to output folder.
Optionally resize the window to the full available page `scrollHeight` and `scrollWidth` to capture the entire page by passing `true` in as the second argument.

```js
I.saveScreenshot('debug.png');
I.saveScreenshot('debug.png', true) //resizes to available scrollHeight and scrollWidth before taking screenshot
```

#### Parameters

-   `fileName` **[string][4]** file name to save.
-   `fullPage` **[boolean][15]** (optional, `false` by default) flag to enable fullscreen screenshot mode. (optional, default `false`)

### type

Types out the given string or the array of keys provided.
_Note:_ Should only be used when using [`fillField`][19] is not an option.

```js
// When passing in a string
I.type('Type this out.');
// When passing in an array
I.type(['T', 'E', 'X', 'T']);
```

#### Parameters

-   `keys`  
-   `key` **([string][4] \| [Array][14]&lt;[string][4]>)** or array of keys to type.
    Type out given array of keys or a string of text

### dragAndDrop

Drag an item to a destination element.

```js
I.dragAndDrop('#dragHandle', '#container');
```

#### Parameters

-   `srcElement` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.
-   `destElement` **([string][4] \| [object][6])** located by CSS|XPath|strict locator.
    Appium: not tested

### dragSlider

Drag the scrubber of a slider to a given position
For fuzzy locators, fields are matched by label text, the "name" attribute, CSS, and XPath.

```js
I.dragSlider('#slider', 30);
I.dragSlider('#slider', -70);
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** located by label|name|CSS|XPath|strict locator.
-   `offsetX` **[number][8]** position to drag. (optional, default `0`)

### grabAllWindowHandles

Get all Window Handles.
Useful for referencing a specific handle when calling `I.switchToWindow(handle)`

```js
const windows = await I.grabAllWindowHandles();
```

### grabCurrentWindowHandle

Get the current Window Handle.
Useful for referencing it when calling `I.switchToWindow(handle)`

```js
const window = await I.grabCurrentWindowHandle();
```

### switchToWindow

Switch to the window with a specified handle.

```js
const windows = await I.grabAllWindowHandles();
// ... do something
await I.switchToWindow( windows[0] );

const window = await I.grabCurrentWindowHandle();
// ... do something
await I.switchToWindow( window );
```

#### Parameters

-   `window`  

### closeOtherTabs

Close all tabs except for the current one.

```js
I.closeOtherTabs();
```

### switchTo

Switches frame or in case of null locator reverts to parent.

```js
I.switchTo('iframe'); // switch to first iframe
I.switchTo(); // switch back to main page
```

#### Parameters

-   `locator` **([string][4]? | [object][6])** (optional, `null` by default) element located by CSS|XPath|strict locator. (optional, default `null`)

### grabNumberOfOpenTabs

Grab number of open tabs.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let tabs = await I.grabNumberOfOpenTabs();
```

Returns **[Promise][13]&lt;[number][8]>** number of open tabs

### scrollPageToTop

Scroll page to the top.

```js
I.scrollPageToTop();
```

### scrollPageToBottom

Scroll page to the bottom.

```js
I.scrollPageToBottom();
```

### grabPageScrollPosition

Retrieves a page scroll position and returns it to test.
Resumes test execution, so **should be used inside an async function with `await`** operator.

```js
let { x, y } = await I.grabPageScrollPosition();
```

Returns **[Promise][13]&lt;[Object][6]&lt;[string][4], any>>** scroll position

### setGeoLocation

Set the current geo location

```js
I.setGeoLocation(121.21, 11.56);
I.setGeoLocation(121.21, 11.56, 10);
```

#### Parameters

-   `latitude` **[number][8]** to set.
-   `longitude` **[number][8]** to set
-   `altitude` **[number][8]** (optional, null by default) to set (optional, default `null`)

### grabGeoLocation

Return the current geo location 
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let geoLocation = await I.grabGeoLocation();
```

Returns **[Promise][13]&lt;{latitude: [number][8], longitude: [number][8], altitude: [number][8]}>** 

### grabElementBoundingRect

Grab the width, height, location of given locator.
Provide `width` or `height`as second param to get your desired prop.
Resumes test execution, so **should be used inside an async function with `await`** operator.

Returns an object with `x`, `y`, `width`, `height` keys.

```js
const value = await I.grabElementBoundingRect('h3');
// value is like { x: 226.5, y: 89, width: 527, height: 220 }
```

To get only one metric use second parameter:

```js
const width = await I.grabElementBoundingRect('h3', 'width');
// width == 527
```

#### Parameters

-   `locator` **([string][4] \| [object][6])** element located by CSS|XPath|strict locator.
-   `prop`  
-   `elementSize` **[string][4]** x, y, width or height of the given element.

Returns **[object][6]** Element bounding rectangle

[1]: http://codecept.io/helpers/WebDriver/

[2]: http://appium.io/

[3]: https://github.com/appium/appium/blob/master/docs/en/writing-running-appium/caps.md

[4]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String

[5]: http://webdriver.io/api/mobile/setNetworkConnection.html

[6]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object

[7]: https://developer.android.com/reference/android/view/KeyEvent.html

[8]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number

[9]: http://webdriver.io/api/mobile/touchAction.html

[10]: http://webdriver.io/api/mobile/swipe.html

[11]: http://webdriver.io/api/mobile/rotate.html

[12]: http://webdriver.io/api/mobile/setImmediateValue.html

[13]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise

[14]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array

[15]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean

[16]: https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView

[17]: https://webdriver.io/docs/timeouts.html

[18]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/undefined

[19]: #fillfield
